@@ -14,17 +14,11 @@
  * limitations under the License.
  */
 
-#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
 /**
  * @file globus_gsi_credential.h
- * Globus GSI Credential Library
+ * @brief Globus GSI Credential Library
  * @author Sam Lang, Sam Meder
- *
- * $RCSfile: globus_gsi_credential.h,v $
- * $Revision: 1.22.10.2 $
- * $Date: 2013/04/15 14:26:56 $
- */
-#endif
+*/
 
 #ifndef GLOBUS_INCLUDE_GLOBUS_GSI_CREDENTIAL_H
 #define GLOBUS_INCLUDE_GLOBUS_GSI_CREDENTIAL_H
@@ -42,10 +36,19 @@
 #include "globus_error_generic.h"
 #include "globus_error_openssl.h"
 
-EXTERN_C_BEGIN
+#ifdef __cplusplus
+extern "C" {
+#endif
 
+#ifndef GLOBUS_GLOBAL_DOCUMENT_SET
 /**
  * @mainpage Globus GSI Credential
+ * @copydoc globus_gsi_credential
+ */
+#endif
+
+/**
+ * @defgroup globus_gsi_credential Globus GSI Credential
  *
  * The Globus GSI Credential library. This library contains functions that
  * provide support for handling X.509 based PKI credentials
@@ -59,6 +62,7 @@ EXTERN_C_BEGIN
 
 /** 
  * @defgroup globus_gsi_credential_activation Activation
+ * @ingroup globus_gsi_credential
  *
  * Globus GSI Credential uses standard Globus module activation and
  * deactivation.  Before any Globus GSI Credential functions are called, the
@@ -100,11 +104,12 @@ globus_module_descriptor_t              globus_i_gsi_credential_module;
 		    s) 
 /**
  * @defgroup globus_gsi_cred_handle Credential Handle Management
+ * @ingroup globus_gsi_credential
  *
  * Create/Destroy/Modify a GSI Credential Handle.
  *
  * Within the Globus GSI Credential Library, all credential operations 
- * require a handle parameter.  Currenlty only one operation may be
+ * require a handle parameter.  Currently only one operation may be
  * in progress at once per credential handle.
  *
  * This section defines operations to create, modify and destroy GSI
@@ -113,6 +118,7 @@ globus_module_descriptor_t              globus_i_gsi_credential_module;
 
 /**
  * @defgroup globus_gsi_cred_handle_attrs Credential Handle Attributes
+ * @ingroup globus_gsi_credential
  *
  * Create/Destroy/Modify GSI Credential Handle Attributes.
  *
@@ -127,6 +133,7 @@ globus_module_descriptor_t              globus_i_gsi_credential_module;
 
 /**
  * @defgroup globus_gsi_cred_operations Credential Operations
+ * @ingroup globus_gsi_credential
  *
  * Read/Write a GSI Credential Handle.
  *
@@ -346,6 +353,8 @@ globus_result_t globus_gsi_cred_handle_attrs_get_search_order(
     globus_gsi_cred_handle_attrs_t      handle_attrs,
     globus_gsi_cred_type_t *            search_order[]);/*{PROXY,USER,HOST}*/
 
-EXTERN_C_END
+#ifdef __cplusplus
+}
+#endif
 
 #endif /* GLOBUS_INCLUDE_GLOBUS_GSI_CREDENTIAL_H */
